﻿using Kivii;
using Kivii.Lims.Entities;
using Kivii.Linq;
using Kivii.Text;
using Njhg.Third.Semir.Entities;
using Njhg.Third.Semir.Transforms;
using System;
using System.Collections.Generic;
using System.Data;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace Njhg.Third.Semir.Extensions
{
    public static class SemirExtension
    {
        /// <summary>
        /// 拉取森马订单
        /// </summary>
        /// <param name="baseUrl"></param>
        /// <param name="token"></param>
        /// <returns></returns>
        internal static SemirOrderResponse PullSemirOrder(this JsonServiceClient _client, string token, DateTime? beginTime = null, DateTime? endTime = null, int length = 200)
        {
            (_client == null).ThrowIfTrue("PullSemirOrder:请传入_client");
            (token.IsNullOrEmpty()).ThrowIfTrue("PullSemirOrder:请传入token");
            //var client = new JsonServiceClient(baseUrl);

            var getWtsListRequest = new SemirOrderGetWtsList();
            getWtsListRequest.token = token;
            getWtsListRequest.length = length;
            if (beginTime != null) getWtsListRequest.begin_time = beginTime.Value;
            if (endTime != null) getWtsListRequest.end_time = endTime.Value;

            var response = _client.Get(getWtsListRequest);
            return response;
        }

        /// <summary>
        /// 创建第三方订单数据
        /// </summary>
        /// <param name="semirOrders"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        internal static List<ThirdReportOrder> AcceptThirdContractOrder(this List<SemirOrder> semirOrders, Guid? ownerKvid = null, string ownerName = null, IDbConnection conn = null)
        {
            (semirOrders.IsNullOrEmpty()).ThrowIfTrue("AcceptThirdContractOrder:请传入semirOrders");
            bool useTransaction = conn == null;//是否启用事务，如果外部未传入conn,启用内部事务
            if (conn == null)
            {
                conn = KiviiContext.GetOpenedDbConnection<ThirdReportOrder>();
            }

            var rtns = new List<ThirdReportOrder>();

            //找出最小创建日期的订单
            var minCreateTime = semirOrders.Min(o => o.CREATED_TIME);
            if (minCreateTime == DateTime.MinValue) minCreateTime = DateTime.Now;
            var queryThirdReportOrders = conn.From<ThirdReportOrder>();
            queryThirdReportOrders.Where(o => o.OperateTime >= minCreateTime & o.ThirdType == typeof(SemirOrder).FullName);
            queryThirdReportOrders.Select(o => o.SerialNumber);
            var thirdReportOrders = conn.Column<string>(queryThirdReportOrders);

            //IDbTransaction trans = null;//事务，如果外部来了Connection，不生成事务
            //if (useTransaction) trans = conn.OpenTransaction();

            try
            {
                foreach (var item in semirOrders)
                {
                    if (thirdReportOrders.Exists(o => o == item.DOC_NO)) continue;

                    var third = new ThirdReportOrder();
                    if (ownerKvid != null) third.OwnerKvid = ownerKvid.Value;
                    if (!ownerName.IsNullOrEmpty()) third.OwnerName = ownerName;
                    third.SerialNumber = item.DOC_NO;
                    third.Name = item.MAT_NAME.Length > 100 ? item.MAT_NAME.Substring(0, 99) : item.MAT_NAME;
                    third.ThirdName = item.BRAND_DISPLAY;
                    third.ThirdType = typeof(SemirOrder).FullName;
                    third.IsAccepted = false;
                    third.IsUploaded = false;
                    third.OperateTime = item.CREATED_TIME;

                    third.ManufacturerName = item.APPLY_ORG_NAME;
                    third.ManufacturerContactName = item.PAYER_CONTACT_PERSON;
                    third.SampleName = item.MAT_NAME.Length > 100 ? item.MAT_NAME.Substring(0, 99) : item.MAT_NAME;
                    third.BatchNumber = item.FG_MAT_CODE;
                    third.Brand = item.BRAND_DISPLAY;
                    //thirdReportOrder.Content = JsonSerializer.SerializeToString(item);
                    third.Metadata = new Dictionary<string, string>();
                    foreach (var propertyInfo in item.GetType().GetProperties())
                    {
                        if (propertyInfo.Name == "ITEMS")
                        {
                            var items = item.ITEMS;
                            if (items.IsNullOrEmpty()) items = new List<SemirOrderDetail>();
                            third.Metadata.Add(propertyInfo.Name, JsonSerializer.SerializeToString(items));
                            continue;
                        }
                        var value = propertyInfo.GetValue(item);
                        third.Metadata.Add(propertyInfo.Name, value == null ? string.Empty : value.ToString());
                    }
                    conn.Insert(third);
                    //trans?.Commit();
                    third.RemoveAllOnlyProperties();
                    rtns.Add(third);
                }
            }
            catch (Exception ex)
            {
                //trans?.Rollback();
                throw ex;
            }

            return rtns;
        }

        internal static bool ConfirmAccept(this JsonServiceClient _client, string token, ThirdReportOrder thirdReportOrder)
        {
            (_client == null).ThrowIfTrue("ConfirmAccept:请传入_client");
            (token.IsNullOrEmpty()).ThrowIfTrue("ConfirmAccept:请传入token");
            var body = new Dictionary<string, string>();
            body.Add("DOC_NO", thirdReportOrder.SerialNumber);
            var response = _client.Post<SemirOrderResponse>($"{Configs.RouteAcceptWts}?token={token}", body);
            if (response.code != 200) return false;
            return true;
        }

        /// <summary>
        /// 自动关联报告,规则:存在不合格的报告不自动关联,只有全合格的才自动关联
        /// </summary>
        /// <param name="third"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        internal static void AutoLinkReports(this ThirdReportOrder third, IDbConnection conn = null)
        {
            third.ThrowIfNull("请传入订单信息!");
            (third.Kvid == Guid.Empty).ThrowIfTrue("请传入订单Kvid信息!");
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Report>();

            //third = conn.SingleById<ThirdContractOrder>(third.Kvid);
            //third.ThrowIfNull("未找到对应订单信息!");
            if (third.IsAccepted) return;

            var queryReports = conn.From<Report>();
            queryReports.Where(o => o.BizId == third.SerialNumber & o.Status >= (int)ReportStatusType.ReportIssue).OrderBy(o => o.ReportId);
            var reports = conn.Select(queryReports);
            if (reports.IsNullOrEmpty()) return;

            var infoJudement = true;
            if (reports.Exists(o => o.TestJudgement == "不合格")) infoJudement = false;

            if (!infoJudement)
            {
                third.ReportIds = "存在不合格报告请手动关联!";
                third.AddOnlyProperties(o => o.ReportIds);
                conn.UpdateOnly(third);
                return;
            }
            //var trans = conn.OpenTransaction();
            try
            {
                third.ReportIds = string.Empty;
                var reportIds = new List<string>();
                foreach (var report in reports)
                {
                    report.BizId = third.SerialNumber;
                    report.AddOnlyProperties(o => o.BizId);
                    report.BizKvid = third.Kvid;
                    report.AddOnlyProperties(o => o.BizKvid);
                    report.BizType = typeof(ThirdReportOrder).FullName;
                    report.AddOnlyProperties(o => o.BizType);
                    conn.UpdateOnly(report);

                    var thirdDetail = new ThirdReportOrderDetail();
                    thirdDetail.ThirdOrderKvid = third.Kvid;
                    thirdDetail.ReportId = report.ReportId;
                    thirdDetail.ReportKvid = report.Kvid;
                    thirdDetail.ThirdType = third.ThirdType;
                    thirdDetail.SerialNumber = third.SerialNumber;
                    thirdDetail.Name = third.Name;
                    thirdDetail.ThirdName = third.ThirdName;
                    thirdDetail.IsUploaded = false;
                    thirdDetail.OperateTime = DateTime.Now;
                    thirdDetail.Metadata = new Dictionary<string, string>();
                    thirdDetail.Metadata = report.Metadata;
                    foreach (var propertyInfo in report.GetType().GetProperties())
                    {
                        if (propertyInfo.Name == "Metadata") continue;
                        try
                        {
                            var value = propertyInfo.GetValue(report);
                            thirdDetail.Metadata.Add(propertyInfo.Name, value == null ? string.Empty : value.ToString());
                        }
                        catch { }

                    }
                    conn.Insert(thirdDetail);
                    reportIds.Add(report.ReportId);
                }

                if (third.Metadata.IsNullOrEmpty()) third.Metadata = new Dictionary<string, string>();
                third.Metadata["Amount"] = reports.Sum(o => o.Amount + o.AmountTest).ToString();
                third.AddOnlyProperties(o => o.Metadata);
                third.ReportIds = string.Join("、", reportIds);
                third.AddOnlyProperties(o => o.ReportIds);
                third.IsAccepted = true;
                third.AddOnlyProperties(o => o.IsAccepted);
                conn.UpdateOnly(third);

                //trans.Commit();
            }
            catch (Exception ex)
            {
                //trans.Rollback();
                throw ex;
            }
        }

        internal static void LinkReports(List<ThirdReportOrder> thirds = null, JsonServiceClient _client = null, string token = null)
        {
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            if (thirds.IsNullOrEmpty())
            {
                thirds = conn.Select<ThirdReportOrder>(o => o.IsAccepted == false && o.IsOffseted == false && o.IsUploaded == false && o.ThirdType == typeof(SemirOrder).FullName);
            }
            if (thirds.IsNullOrEmpty()) return;
            var queryReports = conn.From<Report>();
            queryReports.Where(o => Sql.In(o.BizId, thirds.ConvertAll(p => p.SerialNumber))).OrderBy(o => o.ReportId);
            var allReports = conn.Select(queryReports);
            if (allReports.IsNullOrEmpty()) return;

            foreach (var third in thirds)
            {
                var reports = allReports.Where(o => o.BizId == third.SerialNumber).ToList();
                if (reports.IsNullOrEmpty()) continue;
                if (reports.Exists(o => o.TestJudgement == "不合格"))
                {
                    third.ReportIds = "存在不合格报告请手动关联!";
                    third.AddOnlyProperties(o => o.ReportIds);
                    conn.UpdateOnly(third);
                    continue;
                }
                third.ReportIds = string.Empty;
                var reportIds = new List<string>();
                foreach (var report in reports)
                {
                    report.BizId = third.SerialNumber;
                    report.AddOnlyProperties(o => o.BizId);
                    report.BizKvid = third.Kvid;
                    report.AddOnlyProperties(o => o.BizKvid);
                    report.BizType = typeof(ThirdReportOrder).FullName;
                    report.AddOnlyProperties(o => o.BizType);
                    conn.UpdateOnly(report);

                    var thirdDetail = new ThirdReportOrderDetail();
                    thirdDetail.ThirdOrderKvid = third.Kvid;
                    thirdDetail.ReportId = report.ReportId;
                    thirdDetail.ReportKvid = report.Kvid;
                    thirdDetail.ThirdType = third.ThirdType;
                    thirdDetail.SerialNumber = third.SerialNumber;
                    thirdDetail.Name = third.Name;
                    thirdDetail.ThirdName = third.ThirdName;
                    thirdDetail.IsUploaded = false;
                    thirdDetail.OperateTime = DateTime.Now;
                    thirdDetail.Metadata = new Dictionary<string, string>();
                    thirdDetail.Metadata = report.Metadata;
                    foreach (var propertyInfo in report.GetType().GetProperties())
                    {
                        if (propertyInfo.Name == "Metadata") continue;
                        try
                        {
                            var value = propertyInfo.GetValue(report);
                            thirdDetail.Metadata.Add(propertyInfo.Name, value == null ? string.Empty : value.ToString());
                        }
                        catch { }

                    }
                    conn.Insert(thirdDetail);
                    reportIds.Add(report.ReportId);
                }

                if (third.Metadata.IsNullOrEmpty()) third.Metadata = new Dictionary<string, string>();
                third.Metadata["Amount"] = reports.Sum(o => o.Amount + o.AmountTest).ToString();
                third.AddOnlyProperties(o => o.Metadata);
                third.ReportIds = string.Join("、", reportIds);
                third.AddOnlyProperties(o => o.ReportIds);
                third.IsAccepted = true;
                third.AddOnlyProperties(o => o.IsAccepted);
                conn.UpdateOnly(third);
                if (_client != null && token != null) _client.ConfirmAccept(token, third);
            }
        }

        /// <summary>
        /// 字典类型转化为对象
        /// </summary>
        /// <param name="dic"></param>
        /// <returns></returns>
        public static SemirOrder DicToObject(this Dictionary<string, string> dic)
        {
            var md = new SemirOrder();
            CultureInfo cultureInfo = Thread.CurrentThread.CurrentCulture;
            TextInfo textInfo = cultureInfo.TextInfo;
            foreach (var d in dic)
            {
                var filed = textInfo.ToTitleCase(d.Key);
                try
                {
                    var value = d.Value;
                    var property = md.GetType().GetProperty(filed);
                    if (property == null) continue;
                    if (!property.PropertyType.IsGenericType)
                    {
                        switch (property.Name)
                        {
                            case "ITEMS":
                                property.SetValue(md, JsonSerializer.DeserializeFromString<List<SemirOrderDetail>>(value), null);
                                break;
                            default:
                                property.SetValue(md, string.IsNullOrEmpty(value) ? null : Convert.ChangeType(value, property.PropertyType), null);
                                break;
                        }
                    }
                    else
                    {
                        Type genericTypeDefinition = property.PropertyType.GetGenericTypeDefinition();
                        switch (property.Name)
                        {
                            case "ITEMS":
                                property.SetValue(md, JsonSerializer.DeserializeFromString<List<SemirOrderDetail>>(value), null);
                                break;
                            default:
                                //泛型Nullable<>
                                if (genericTypeDefinition == typeof(Nullable<>))
                                {
                                    property.SetValue(md, string.IsNullOrEmpty(value) ? null : Convert.ChangeType(value, Nullable.GetUnderlyingType(property.PropertyType)), null);
                                }
                                break;
                        }
                    }
                }
                catch (Exception ex)
                {
                    throw ex;
                }
            }
            return md;
        }
    }
}
